@@ -2,15 +2,10 @@
 # -*- coding: utf-8 -*-
 
 """Init file"""
-from .read_db import db_reader
-from .water_eos import iapws95, ZhangDuan, water_dielec, readIAPWS95data, convert_temperature
+
+from .pygcc_utils import *
+from .read_db import db_reader, dbaccess_modify
+from .water_eos import iapws95, ZhangDuan, water_dielec, readIAPWS95data, convert_temperature, Driesner_NaCl, concentration_converter
 from .species_eos import heatcap, supcrtaq, Element_counts
 from .solid_solution import solidsolution_thermo
 from .clay_thermocalc import calclogKclays, MW
-from .pygcc_utils import *
-
-# read version from installed package
-from importlib_metadata import version
-__version__ = version("pygcc")
-
-
